/***************************************************************************/
// Program to export WGI aggregate and source data into different formats
// for dissemination at www.govindicators.org
/***************************************************************************/
// Called from:  wgicode1.do 
// Input: 	resultsdata.dta, containing all the individual indicators and aggregate indicators
//			and all the model parameters
// Output: 	Files aggindstableau.xlsx and individualindstablea.xlsx with data to export to Tableau database
//			Files parameterestimates.dta and parameterestimates.xlsx containing the estimated parameters of UCM (alpha, beta, sigma) in long format stacking indicators/sources/years 
//			File wgisourceweights.xlsx with average weights applied to data sources, by indicator and year
// 			Files wgidataset.xlsx and wgidataset.dta containing basic version of WGI data (only six aggregate indicators, by country and year)	
//			Files wgidataset_with_sourcedata.xlsx and wgidataset_with_sourcedata.dta containing extended version of dataset including all source variables
// Codes called: NONE
/***************************************************************************/

/******************************************************************************/
// 1. Export data to format for Tableau 
/******************************************************************************/
// Export aggregate indicators to Excel, single sheet, rectangular format for all 250 country codes (i.e. including codes for which there is no data)
preserve
keep cid va* pv* ge* rq* rl* cc*
drop vab*
drop *rep
drop ccr*cc* ccr*va* ccr*rl*
drop *mean* *sd*
rename *, upper
export excel using "$filepath\output\aggindstableau.xlsx", replace firstrow(var) missing("#N/A")
restore
// Export individual indicators to Excel, rectangular format for all 250 country codes (i.e. including codes for which there is no data)
preserve
foreach src in $sourcelist{
	foreach ind in $indicatorlist{
		foreach per in $periodlist{
			cap rename `src'`per'`ind'1 `src'`ind'`per'_keep
			cap rename `src'`per'`ind'2 `src'`ind'`per'_keep
		}
	}
}
keep cid code countryname *_keep
order cid code countryname
rename *_keep *
rename *, upper 
export excel using "$filepath\output\individualindstableau.xlsx", replace firstrow(var) missing("#N/A")
restore

/******************************************************************************/
// 2. Export parameter estimates in "long" format stacking sources and years
/******************************************************************************/
preserve
// Keep only first row of dataset (parameter estimates are repeated in every row)
keep if cid==1
// Keep only variables containing parameters
keep cid *alpha *beta *sigma
// Rename parameters more conveniently for reshaping (parameter name as stub)
foreach src in $sourcelist{
	foreach ind in $indicatorlist{
		foreach per in $periodlist{
			cap rename `src'`per'`ind'1alpha alpha`src'`per'`ind'
			cap rename `src'`per'`ind'1beta beta`src'`per'`ind'
			cap rename `src'`per'`ind'1sigma sigma`src'`per'`ind'
			cap rename `src'`per'`ind'2alpha alpha`src'`per'`ind'
			cap rename `src'`per'`ind'2beta beta`src'`per'`ind'
			cap rename `src'`per'`ind'2sigma sigma`src'`per'`ind'
		}
	}
}
reshape long alpha beta sigma, i(cid) j(srcperind) string
// Generate separate variables for source, period and indicator row identifiers
gen source=substr(srcperind,1,3)
gen per=substr(srcperind,4,2)
gen indicator=substr(srcperind,6,2)
gen xx="20"
replace xx="19" if per=="98" | per=="96"
gen year=xx+per
gen srcindyr=source+indicator+year
destring year, replace
// Label variables and export
drop xx cid per srcperind
sort source indicator year
order srcindyr year indicator source alpha beta sigma
label variable srcindyr "Unique identifier for each row of dataset (source+indicator+year)"
label variable year "Year"
label variable indicator "WGI indicator code va/pv/ge/rq/rl/cc"
label variable source "WGI data source code"
label variable alpha "Estimate of alpha"
label variable beta "Estimate of beta"
label variable sigma "Estimate of sigma"
// Replace a few cases of negative values of sigma with its absolute values
// This is innocuous, has to do with how the UCM code estimates sigma x beta first and then
// divides by beta (which can be negative) to retrieve sigma.  In any case only sigma^2 enters
// WGI calculations, so this change is only cosmetic 
replace sigma=abs(sigma)
save "$filepath\output\wgiparameterestimates", replace
export excel using "$filepath\output\wgiparameterestimates.xlsx", replace firstrow(var) missing("..")
restore

/******************************************************************************/
// 3. Create "long" version of WGI dataset including the aggregate indicators,
// source data, parameters, and weights
/******************************************************************************/
drop *mean *sd *load *rep
// Rename source data and parameters more conveniently for reshaping (data source code as stub)
foreach src in $sourcelist{
	foreach ind in $indicatorlist{
		foreach per in $periodlist{
			cap rename `src'`per'`ind'1 `src'`ind'`per'
			cap rename `src'`per'`ind'2 `src'`ind'`per'
			cap rename `src'`per'`ind'1alpha `src'_alpha`ind'`per'
			cap rename `src'`per'`ind'1beta `src'_beta`ind'`per'
			cap rename `src'`per'`ind'1sigma `src'_sigma`ind'`per'
			cap rename `src'`per'`ind'2alpha `src'_alpha`ind'`per'
			cap rename `src'`per'`ind'2beta `src'_beta`ind'`per'
			cap rename `src'`per'`ind'2sigma `src'_sigma`ind'`per'
		}
	}
}

// Rename aggregate indicators more conveniently for reshaping (variable name as stub)
// Note variables "scalemean" and "scalesd" contain the factors by 
// which each aggregate indicator is standardized to ensure that
// published data has zero mean and unit standard deviation.  
// Need these as inputs to the Excel file that demonstrates the
// calculations
foreach ind in $indicatorlist{
	foreach per in $periodlist{
		rename `ind'e`per' estimate`ind'`per'
		rename `ind's`per' stddev`ind'`per'
		rename `ind'n`per' nsource`ind'`per'
		rename `ind'r`per' pctrank`ind'`per'
		rename `ind'l`per' pctranklower`ind'`per'
		rename `ind'u`per' pctrankupper`ind'`per'
		rename `ind'mean`per' scalemean`ind'`per'
		rename `ind'sd`per' scalesd`ind'`per'
	}
}

// Determine set of countries with data for at least one of the aggregate indicators, drop all others
gen smpl=0
foreach ind in $indicatorlist{
	foreach per in $periodlist{
		qui replace smpl=smpl+1 if estimate`ind'`per'~=.
		}
	}
keep if smpl>0
su smpl  // check to be sure number of countries is stable from year to year, 214 as of 2024 update
drop smpl

// Build list of stubs for reshaping.  Start with stubs for core aggregate indicators
// Then loop over sources to add on stubs of variable and parameters for each data source
global stublist "estimate stddev nsource pctrank pctranklower pctrankupper scalemean scalesd"
foreach src in $sourcelist{
	global stublist "$stublist `src' `src'_alpha `src'_beta `src'_sigma"
}

// Reshape data into long format stacking countries, aggregate indicators and years vertically
reshape long $stublist, i(cid) j(indper) string 

// Split out separate variables to mark indicators and periods
gen indicator=substr(indper,1,2)
gen per=substr(indper,3,2)
gen xx="20"
replace xx="19" if per=="98" | per=="96"
gen year=xx+per
gen codeindyr=code+indicator+year
destring year, replace
drop xx cid per indper
sort code indicator year
order codeindyr code countryname year indicator estimate stddev nsource pctrank pctranklower pctrankupper $sourcelist scalemean scalesd

// Replace parameter vectors to missing if corresponding source data is missing
foreach src in $sourcelist{
	foreach param in alpha beta sigma{
		replace `src'_`param'=. if `src'==. 
	}
}

// Generate measure of precision for each data source (sigma^-2) used to construct source weights
foreach src in $sourcelist{
	gen `src'_prec=`src'_sigma^-2
}
// Generate weights assigned to each data source: sigma^-2/(1+sum(sigma^-2))
egen xx=rowtotal(*_prec), missing
foreach src in $sourcelist{
	gen `src'_weight=`src'_sigma^-2/(1+xx)
}
drop xx

// Label variables in long version of dataset
label variable code "ISO 3-letter country code"
label variable countryname "Country Name"
label variable year "Year"
label variable codeindyr "Unique identifier for each row in dataset (country code + WGI indicator + year)"
label variable indicator "WGI Aggregate Indicator va/pv/ge/rq/rl/cc"
label variable estimate "Estimate of Governance (approx. -2.5 to 2.5)"
label variable stddev "Standard Deviation of Estimate of Governance"
label variable nsource "Number of Data Sources"
label variable pctrank "Percentile Rank (0-100)"
label variable pctranklower "Percentile Rank of Lower Bound of 90% Confidence Interval (0-100)"
label variable pctrankupper "Percentile Rank of Upper Bound of 90% Confidence Interval (0-100)"
global adb_label "African Development Bank Country Policy Assessment"
global afr_label "Afrobarometer"
global asd_label "Asian Development Bank Country Policy Assessment"
global bps_label "World Bank Business Environment and Enterprise Performance Survey"
global bti_label "Bertlesmann Transformation Index"
global ccr_label "Freedom House Countries at the Crossroads"
global ebr_label "European Bank for Reconstruction and Development Transition Indicators"
global eiu_label "Economist Intelligence Unit Country Risk Service"
global eqi_label "European Quality of Governance Survey"
global frh_label "Freedom House Freedom in the World, Freedom of the Net, Nations in Transit"
global gcb_label "Transparency International Global Corruption Barometer"
global gcs_label "World Economic Forum Executive Opinion Survey"
global gii_label "Africa Integrity Indicators"
global gwp_label "Gallup World Poll"
global her_label "Heritage Foundation"
global hrm_label "Human Rights Measurement Initiative"
global hum_label "Human Rights Database and Political Terror Scale"
global ifd_label "International Fund for Agriculture and Development Country Policy Assessment"
global ijt_label "Crisis24 Country Security Assessment Ratings"
global ipd_label "Institutional Profiles Database"
global irp_label "African Electoral Integrity Indicators"
global lbo_label "Latinobarometer"
global msi_label "Vibrant Information Barometer"
global obi_label "Open Budget Index"
global pia_label "World Bank Country Policy and Institutional Assessment"
global prc_label "Political Risk Consultancy Corruption in Asia"
global prs_label "Political Risk Services International Country Risk Guide"
global rsf_label "Reporters Without Borders"
global tpr_label "US State Department Trafficking in Persons Report"
global vab_label "Americas Barometer"
global vdm_label "Varieties of Democracy"
global wbs_label "World Bank Enterprise Surveys"
global wcy_label "World Competitiveness Yearbook"
global wjp_label "World Justice Project"
global wmo_label "S&PGlobal Country Risk Service"
foreach src in $sourcelist{
	label variable `src' "$`src'_label"
	label variable `src'_weight "$`src'_label: Weight"
}
label variable scalemean "Mean of governance before rescaling to mean=zero and sd=1"
label variable scalesd "Standard deviation of governance before rescaling to mean=zero and sd=1"

sort year indicator countryname
// Save compact version of WGI dataset containing aggregate indicators only
preserve
keep codeindyr code countryname year indicator estimate stddev nsource pctrank pctranklower pctrankupper 
save "$filepath\output\wgidataset.dta", replace
export excel using "$filepath\output\wgidataset.xlsx", replace firstrow(var) missing("..")
restore
// Save extended version of WGI dataset including aggregate indicators and source data
preserve
keep codeindyr code countryname year indicator estimate stddev nsource pctrank pctranklower pctrankupper $sourcelist scalemean scalesd
save "$filepath\output\wgidataset_with_sourcedata.dta", replace
export excel using "$filepath\output\wgidataset_with_sourcedata.xlsx", replace firstrow(var) missing("..")
restore

/******************************************************************************/
// 4. Export table of weights (average across countries by indicator and year)
/******************************************************************************/
preserve
keep year ind *_weight
collapse (mean) *_weight, by(year ind)
sort ind year
label variable year "Year"
foreach src in $sourcelist{
	label variable `src'_weight "Average Weight: $`src'_label"
}
save "$filepath\output\wgisourceweights", replace
export excel using "$filepath\output\wgisourceweights.xlsx", replace firstrow(var) missing("..")
restore

/******************************************************************************/
// End of code
/******************************************************************************/