/***************************************************************************/
// Program to Estimate Worldwide Governance Indicators (WGI) 
// Version:  August 21 2024 
/***************************************************************************/
// This is a master program that calls several individual programs to do
// the estimation of the WGI given the source data from the individual
// indicators.  The overall structure of the programs called by this 
// program is as follows:
//
// importmerge1.do -- imports the individual indicators data from the the source-specific Excel files.  This program calls a separate
//					.do file for each data source and executes it to retrieve the data from each year tab of the corresponding source-specific
//					Excel file.  These are then merged into a combined dataset with the individual indicators for each data source called
//					sourcedata.dta
//
// estimate1.do -- loops over all the aggregate indicators and time periods.
//					For each indicator-year, it retrieves the corresponding data from sourcedata.dta
//					estimates the Unobserved Components Model (UCM) for representative sources, and generates the
//					corresponding aggregate indicator using data for the non-representative
//					sources as well.  After completing this loop, it also calculates the 
//					percentile ranks of the indicators and upper and lower bounds of 90%
//					confidence intervals. This program calls three sub-programs to do 
//					these three tasks:
//						ucmcode1.do (estimates Unobserved Components Model (UCM))
//							Note: requires as input an Excel file with the master list of country names and codes (countrycodes.xlsx)
//						nonrepcode1.do (incorporates nonrepresentative sources)
//						rankcalc1.do (compute percentile ranks)
//					Key output is "resultsdata.dta" which stores all the individual indicators, aggregate indicators, and parameter estimates
//
//	exporter1.do -- uses output generated by two previous codes to generate a series of Excel and Stata versions of the dataset for posting on the WGI website 
//
// User should set global filepath below to location where replication package is downloaded
/***************************************************************************/

/***************************************************************************/
// 1. Preliminaries
/***************************************************************************/
clear
clear matrix
clear mata
set more off
set maxvar 100000
set matsize 5000
set varabbrev on

// Set filepath
global filepath "C:\Users\wb614536\Downloads\FR_WLD_2024_186\reproducibility package"		
cd "$filepath"

// Globally-defined lists used throughout code (update source data lists if new data sources are added, and period list with each new year)
// All data sources 
global sourcelist "adb afr asd bps bti ccr ebr eiu eqi frh gcb gcs gii gwp her hum hrm ifd ijt ipd irp lbo msi obi pia prc prs rsf tpr vab vdm wbs wcy wjp wmo"
// Representative and non-representative data sources (note that importmerg1.do contains a few manual over-rides of this classification in certain years)
global repsourcelist "eiu frh gcs gwp her hum ijt ipd prs rsf tpr vdm wmo"
global nonrepsourcelist "adb afr asd bps bti ccr ebr eqi gcb gii hrm ifd irp lbo msi obi pia prc vab wbs wcy wjp"
// List of six aggregate indicators
global indicatorlist "va pv ge rq rl cc" 
// List of time periods 
global periodlist "96 98 00 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23" 		
/***************************************************************************/


/***************************************************************************/
// 2.  Import and merge datasets from source-specific Excel files 
// Retrieves data from source-specific Excel files, one for each source
// using source-specific .do files.  Excel files containing data and source-
// specific .do files are in $filepath\sourcedata and $filepath\sourcedofiles
// Generates output sourcedata.dta containing data from individual data sources
// used to estimate the six aggregate WGI indicators
// This takes 2-3 minutes to run.  Only needs to be re-run if source data changes
/***************************************************************************/
run importmerge1.do

/***************************************************************************/
// 3. Call estimation code 
// Uses as input source data in sourcedata.dta
// Generates output resultsdata.dta containing all source data, 
// parameter estimates, and aggregate indicators. 
/***************************************************************************/
clear
use sourcedata
run estimate1.do

/***************************************************************************/
// 3. Export the aggregate indicators, source data, and parameters in
// different formats used to populate the WGI website and documentation.
// Uses resultsdata.dta as input
/***************************************************************************/
clear
use resultsdata
run exporter1.do

/***************************************************************************/
// End of Code
/***************************************************************************/








